import { Component, Injectable, NgModule } from '@angular/core';
import { ActivatedRoute, Router, RouterModule } from '@angular/router';
import { Subject } from 'rxjs/Subject';
import { Log, UserManager } from 'oidc-client';

class OidcUser {
    /**
     * @param {?=} identityToken
     * @param {?=} sessionState
     * @param {?=} accessToken
     * @param {?=} tokenType
     * @param {?=} scope
     * @param {?=} profile
     * @param {?=} expiresAt
     * @param {?=} state
     * @param {?=} identityTokenData
     * @param {?=} accessTokenData
     */
    constructor(identityToken, sessionState, accessToken, tokenType, scope, profile, expiresAt, state, identityTokenData, accessTokenData) {
        this.identityToken = identityToken;
        this.sessionState = sessionState;
        this.accessToken = accessToken;
        this.tokenType = tokenType;
        this.scope = scope;
        this.profile = profile;
        this.expiresAt = expiresAt;
        this.state = state;
        this.identityTokenData = identityTokenData;
        this.accessTokenData = accessTokenData;
    }
    /**
     * @return {?}
     */
    get expiresIn() {
        if (this.expiresAt) {
            let /** @type {?} */ now = parseInt(Date.now() / 1000 + '');
            return this.expiresAt - now;
        }
        return 0;
    }
    /**
     * @return {?}
     */
    get scopes() {
        return (this.scope || "").split(" ");
    }
    /**
     * @return {?}
     */
    get expired() {
        let /** @type {?} */ expiresIn = this.expiresIn;
        if (expiresIn !== undefined) {
            return expiresIn <= 0;
        }
        return true;
    }
    /**
     * @return {?}
     */
    get authenticated() {
        return this.identityToken && !this.expired;
    }
    /**
     * @param {?} claim
     * @return {?}
     */
    hasIdentityClaim(claim) {
        return this.identityTokenData && this.identityTokenData[claim.name] === claim.value;
    }
    /**
     * @param {?} claim
     * @return {?}
     */
    hasAccessClaim(claim) {
        return this.accessTokenData && this.accessTokenData[claim.name] === claim.value;
    }
}
class OidcProfile {
}
class ClientOidcSettings {
    /**
     * @param {?} authorityUrl
     * @param {?} clientId
     * @param {?} responseType
     * @param {?} scopes
     * @param {?=} loginRoutePath
     * @param {?=} logoutRoutePath
     * @param {?=} errorRoutePath
     */
    constructor(authorityUrl, clientId, responseType, scopes, loginRoutePath, logoutRoutePath, errorRoutePath) {
        this.authorityUrl = authorityUrl;
        this.clientId = clientId;
        this.responseType = responseType;
        this.scopes = scopes;
        this.loginRoutePath = loginRoutePath;
        this.logoutRoutePath = logoutRoutePath;
        this.errorRoutePath = errorRoutePath;
    }
}

class UserManagerSettings {
    /**
     * @param {?} clientOidcSettings
     * @param {?} loadUserInfo
     * @param {?} redirectUri
     * @param {?} postLogoutRedirectUri
     */
    constructor(clientOidcSettings, loadUserInfo, redirectUri, postLogoutRedirectUri) {
        this.authority = clientOidcSettings.authorityUrl;
        this.client_id = clientOidcSettings.clientId;
        this.response_type = clientOidcSettings.responseType;
        this.scope = clientOidcSettings.scopes;
        this.login_client_url = clientOidcSettings.loginRoutePath || "/";
        this.logout_client_url = clientOidcSettings.logoutRoutePath || "/";
        this.error_client_url = clientOidcSettings.errorRoutePath || "/";
        this.loadUserInfo = loadUserInfo;
        this.redirect_uri = redirectUri;
        this.post_logout_redirect_uri = postLogoutRedirectUri;
    }
}
class Claim {
    /**
     * @param {?} name
     * @param {?} value
     */
    constructor(name, value) {
        this.name = name;
        this.value = value;
    }
}

let StorageKeys = {
    UserManagerSettings: "oidc.userManagerSettings",
    User: "oidc.user"
};

class StorageService {
    constructor() {
        this.storage = sessionStorage; // localStorage;
    }
    /**
     * @param {?} key
     * @return {?}
     */
    retrieve(key) {
        let /** @type {?} */ item = this.storage.getItem(key);
        if (item && item !== 'undefined') {
            return JSON.parse(this.storage.getItem(key));
        }
        return;
    }
    /**
     * @param {?} key
     * @param {?} value
     * @return {?}
     */
    store(key, value) {
        this.storage.setItem(key, JSON.stringify(value));
    }
    /**
     * @param {?} key
     * @return {?}
     */
    clear(key) {
        this.storage.removeItem(key);
    }
}
StorageService.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
StorageService.ctorParameters = () => [];

class TokenService {
    /**
     * @param {?} token
     * @return {?}
     */
    getDataFromToken(token) {
        let /** @type {?} */ data = {};
        if (typeof token !== 'undefined') {
            let /** @type {?} */ encoded = token.split('.')[1];
            if (encoded) {
                data = JSON.parse(this.urlBase64Decode(encoded));
            }
        }
        return data;
    }
    /**
     * @param {?} str
     * @return {?}
     */
    urlBase64Decode(str) {
        let /** @type {?} */ output = str.replace('-', '+').replace('_', '/');
        switch (output.length % 4) {
            case 0:
                break;
            case 2:
                output += '==';
                break;
            case 3:
                output += '=';
                break;
            default:
                throw 'Illegal base64url string!';
        }
        return window.atob(output);
    }
}
TokenService.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
TokenService.ctorParameters = () => [];

class UserManagerService {
    /**
     * @param {?} storageService
     * @param {?} tokenService
     */
    constructor(storageService, tokenService) {
        this.storageService = storageService;
        this.tokenService = tokenService;
    }
    /**
     * @return {?}
     */
    get userManager() {
        if (this.userManagerSetting) {
            return new UserManager(this.userManagerSetting);
        }
        else {
            Log.error("UserManagerSettings are not defined");
            return null;
        }
    }
    /**
     * @return {?}
     */
    get userManagerSetting() {
        return this.storageService.retrieve(StorageKeys.UserManagerSettings);
    }
    /**
     * @param {?} user
     * @return {?}
     */
    storeUser(user) {
        this.storageService.store(StorageKeys.User, user);
    }
    /**
     * @return {?}
     */
    clearUser() {
        this.storageService.clear(StorageKeys.User);
        this.userManager.clearStaleState().then(function () {
            Log.info('clearStateState success');
        }).catch(function (e) {
            Log.error('clearStateState error', e.message);
        });
    }
    /**
     * @return {?}
     */
    getUser() {
        let /** @type {?} */ user = this.storageService.retrieve(StorageKeys.User);
        if (!user) {
            return new OidcUser();
        }
        let /** @type {?} */ oidcUser = new OidcUser(user.id_token, user.session_state, user.access_token, user.token_type, user.scope, user.profile, user.expires_at, user.state);
        oidcUser.identityTokenData = this.tokenService.getDataFromToken(oidcUser.identityToken);
        oidcUser.accessTokenData = this.tokenService.getDataFromToken(oidcUser.accessToken);
        return oidcUser;
    }
    /**
     * @param {?} clientOidcSettings
     * @return {?}
     */
    storeUserManagerSettings(clientOidcSettings) {
        let /** @type {?} */ redirectUri = `${location.origin}/oidc`;
        let /** @type {?} */ postLogoutRedirectUri = location.origin + '/' + clientOidcSettings.logoutRoutePath;
        let /** @type {?} */ userManagerSettings = new UserManagerSettings(clientOidcSettings, false, redirectUri, postLogoutRedirectUri);
        this.storageService.store(StorageKeys.UserManagerSettings, userManagerSettings);
    }
}
UserManagerService.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
UserManagerService.ctorParameters = () => [
    { type: StorageService, },
    { type: TokenService, },
];

class UrlService {
    /**
     * @return {?}
     */
    get hasHash() {
        return window.location.hash ? true : false;
    }
}
UrlService.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
UrlService.ctorParameters = () => [];

class OidcService {
    /**
     * @param {?} userManagerService
     * @param {?} router
     * @param {?} urlService
     */
    constructor(userManagerService, router, urlService) {
        this.userManagerService = userManagerService;
        this.router = router;
        this.urlService = urlService;
        this.authenticationSource = new Subject();
        this.onUserDataLoaded = this.authenticationSource.asObservable();
    }
    /**
     * @param {?} clientOidcSettings
     * @return {?}
     */
    init(clientOidcSettings) {
        Log.logger = console;
        this.userManagerService.storeUserManagerSettings(clientOidcSettings);
    }
    /**
     * @return {?}
     */
    signin() {
        if (this.urlService.hasHash) {
            return;
        }
        if (this.userManagerService.userManager) {
            this.userManagerService.userManager.signinRedirect()
                .then(() => Log.info('signinRedirect done'))
                .catch((err) => Log.error(err));
        }
    }
    /**
     * @return {?}
     */
    signinCallback() {
        let /** @type {?} */ $this = this;
        if (this.userManagerService.userManager) {
            this.userManagerService.userManager.signinRedirectCallback()
                .then((user) => {
                Log.info('signed in', user);
                $this.userManagerService.storeUser(user);
                $this.authenticationSource.next($this.userManagerService.getUser());
                $this.router.navigate([$this.userManagerService.userManagerSetting.login_client_url]);
            }).catch((err) => Log.error(err));
        }
    }
    /**
     * @return {?}
     */
    signout() {
        let /** @type {?} */ $this = this;
        if (this.userManagerService.userManager) {
            this.userManagerService.userManager.signoutRedirect({ id_token_hint: this.user.identityToken })
                .then((resp) => {
                Log.info('signed out', resp);
                $this.userManagerService.clearUser();
                $this.authenticationSource.next(this.userManagerService.getUser());
                $this.router.navigate(['app']);
            }).catch((err) => Log.error(err));
        }
    }
    ;
    /**
     * @return {?}
     */
    get user() {
        return this.userManagerService.getUser();
    }
}
OidcService.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
OidcService.ctorParameters = () => [
    { type: UserManagerService, },
    { type: Router, },
    { type: UrlService, },
];

class OidcComponent {
    /**
     * @param {?} userManagerService
     * @param {?} oidcService
     * @param {?} router
     * @param {?} route
     */
    constructor(userManagerService, oidcService, router, route) {
        this.userManagerService = userManagerService;
        this.oidcService = oidcService;
        this.router = router;
        this.route = route;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        let /** @type {?} */ $this = this;
        if (window.location.hash) {
            $this.oidcService.signinCallback();
        }
        else {
            $this.route.queryParams.subscribe((queryParam) => {
                let /** @type {?} */ errMsg = queryParam['errMsg'];
                $this.router.navigate([this.userManagerService.userManagerSetting.error_client_url, errMsg]);
            });
        }
    }
}
OidcComponent.decorators = [
    { type: Component, args: [{
                selector: 'oidc',
                template: ''
            },] },
];
/**
 * @nocollapse
 */
OidcComponent.ctorParameters = () => [
    { type: UserManagerService, },
    { type: OidcService, },
    { type: Router, },
    { type: ActivatedRoute, },
];

const OidcRoutes = [
    {
        path: 'oidc',
        component: OidcComponent
    },
];

class OidcModule {
}
OidcModule.decorators = [
    { type: NgModule, args: [{
                imports: [RouterModule.forRoot(OidcRoutes)],
                declarations: [OidcComponent],
                providers: [UserManagerService, OidcService, StorageService, TokenService, UrlService],
                exports: [OidcComponent, RouterModule]
            },] },
];
/**
 * @nocollapse
 */
OidcModule.ctorParameters = () => [];

/**
 * Generated bundle index. Do not edit.
 */

export { ClientOidcSettings, Claim, OidcUser, OidcProfile, OidcService, Log, OidcModule, OidcComponent as ɵf, OidcRoutes as ɵe, UserManagerService as ɵa, StorageService as ɵb, TokenService as ɵc, UrlService as ɵd };
//# sourceMappingURL=angular-oidc-lib.js.map
